#include "common/AT91SAM7S256.h"
#include "common/lib_AT91SAM7S256.h"
#include "spi.h"
#include "common.h"
#include <stdint.h>
#include <stdio.h>

AT91PS_SPI pSPI      = AT91C_BASE_SPI;
AT91PS_PIO pPIOA     = AT91C_BASE_PIOA;
AT91PS_PMC pPMC      = AT91C_BASE_PMC;
AT91PS_PDC pPDC_SPI  = AT91C_BASE_PDC_SPI;


void spiCsSelect(u08 cs) 
{
	pSPI->SPI_MR |=  (unsigned int)(0xf)<<16 ; // clear PCS
	pSPI->SPI_MR &=  (0xFFF0FFFF | (unsigned int)(cs<<16)); // set PCS
}




void spiSetSpeed(u08 speed, u08 csr)
{
	unsigned long reg;
	//AT91PS_SPI pSPI      = AT91C_BASE_SPI;

	if ( speed < SPI_SCBR_MIN ) speed = SPI_SCBR_MIN;
	//if ( speed > 1 ) speed &= 0xFE;

	reg = pSPI->SPI_CSR[csr];
	reg = ( reg & ~(AT91C_SPI_SCBR) ) | ( (unsigned long)speed << 8 );
	pSPI->SPI_CSR[csr] = reg;
}

void init_spi()
{	
	pPIOA->PIO_PDR = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK; //  | NCPS_PDR_BIT;
	// set pin-functions in PIO Controller
	pPIOA->PIO_ASR = AT91C_PA12_MISO | AT91C_PA13_MOSI | AT91C_PA14_SPCK; 

	// set chip-select as output high (unselect card)
	pPIOA->PIO_PER  = NCPS_ASR_BIT; // enable PIO of CS-pin
	pPIOA->PIO_SODR = NCPS_ASR_BIT; // set
	pPIOA->PIO_OER  = NCPS_ASR_BIT; // output

	// enable peripheral clock for SPI ( PID Bit 5 )
	pPMC->PMC_PCER = ( (u16) 1 << AT91C_ID_SPI ); // n.b. IDs are just bit-numbers

	// SPI enable and reset
	pSPI->SPI_CR = AT91C_SPI_SPIEN | AT91C_SPI_SWRST;

	// SPI mode: master, fixed periph. sel., FDIV=0, fault detection disabled
	pSPI->SPI_MR  = AT91C_SPI_MSTR | AT91C_SPI_PS_FIXED | AT91C_SPI_MODFDIS;

	// set PCS for fixed select
	pSPI->SPI_MR &= 0xFFF0FFFF; // clear old PCS - redundant (AT91lib)
	pSPI->SPI_MR |= ( (ETH_CS<<16) & AT91C_SPI_PCS ); // set PCS

	
	pSPI->SPI_CSR[ETH_CSR] = AT91C_SPI_CPOL | AT91C_SPI_NCPHA | AT91C_SPI_BITS_8 | AT91C_SPI_CSAAT ; //spi Mode 0
	pSPI->SPI_CSR[MMC_CSR] = AT91C_SPI_CPOL | AT91C_SPI_BITS_8; // ok

	// slow during init
	spiSetSpeed(0xFE,MMC_CSR); 
	spiSetSpeed(0xFE,ETH_CSR);
	// enable
	pSPI->SPI_CR = AT91C_SPI_SPIEN;

	/* enable automatic chip-select */
	pPIOA->PIO_ODR  = NCPS_ASR_BIT; // input
	pPIOA->PIO_CODR = NCPS_ASR_BIT; // clear
	// disable PIO from controlling the CS pin (=hand over to SPI)
	pPIOA->PIO_PDR = NCPS_PDR_BIT;
	// set pin-functions in PIO Controller
	pPIOA->PIO_ASR = NCPS_ASR_BIT;
	pPIOA->PIO_BSR = NCPS_BSR_BIT;

}

